/**
 * @file bootloader.c
 *
 * @brief This file contains bootloader routines for the USBlini firmware
 *        project.
 *
 * @author Thomas Fischl (https://www.fischl.de/contact.html)
 * @copyright (c) 2021 Thomas Fischl
 * 
 * This file is part of the USBlini firmware project and is copyrighted by the
 * authors listed above. It is free for private or educational non-commercial use
 * (for a commercial license please contact the authors). It may not be
 * redistributed without the prior written consent of the authors.
 *
 * It is provided "as is" without warranty of any kind, either express or implied,
 * including without limitation any implied warranties of condition, uninterrupted
 * use, merchantability, fitness for a particular purpose, or non-infringement. In
 * no event shall the authors be liable for any direct or indirect damages arising
 * in any way out of the use of it.
 *
 */

#include "usblini.h"
#include "clock.h"
#include "usb.h"
#include "bootloader.h"

#define BOOTLOADER_STATE_OFF 0
#define BOOTLOADER_STATE_TRIGGERED 1
#define BOOTLOADER_STATE_SHUTDOWNAPP 2

#define BOOTLOADER_TIMEOUT_MS 500

uint8_t bootloader_state = BOOTLOADER_STATE_OFF;
uint16_t bootloader_timestamp;

/**
 * @brief Trigger start of bootloader
 */
void bootloader_triggerStart(void) {
    bootloader_state = BOOTLOADER_STATE_TRIGGERED;
    bootloader_timestamp = clock_getMSTicker();
    
}

/**
 * @brief Handle periodic task. Check if bootloader start was triggered, then
 * initiate shutdown and jump to bootloader.
 */
void bootloader_process(void) {
    
    switch (bootloader_state) {
        
        case BOOTLOADER_STATE_OFF:
            return;
            
        case BOOTLOADER_STATE_TRIGGERED:
            if (clock_diff(bootloader_timestamp) > BOOTLOADER_TIMEOUT_MS) {
                bootloader_state = BOOTLOADER_STATE_SHUTDOWNAPP;
                bootloader_timestamp = clock_getMSTicker();
                
                usb_shutdown();
            }
            break;
            
        case BOOTLOADER_STATE_SHUTDOWNAPP:
            if (clock_diff(bootloader_timestamp) > BOOTLOADER_TIMEOUT_MS) {
                
                // jump to bootloader
                asm ("MOVLP 0x00");
                asm ("GOTO 0x30");                
            }
            break;
    }        
    
}
