/**
 * @file main.c
 *
 * @brief This file contains the main routine with application entry point for
 *        the USBlini firmware project.
 *
 * @author Thomas Fischl (https://www.fischl.de/contact.html)
 * @copyright (c) 2021 Thomas Fischl
 * 
 * This file is part of the USBlini firmware project and is copyrighted by the
 * authors listed above. It is free for private or educational non-commercial use
 * (for a commercial license please contact the authors). It may not be
 * redistributed without the prior written consent of the authors.
 *
 * It is provided "as is" without warranty of any kind, either express or implied,
 * including without limitation any implied warranties of condition, uninterrupted
 * use, merchantability, fitness for a particular purpose, or non-infringement. In
 * no event shall the authors be liable for any direct or indirect damages arising
 * in any way out of the use of it.
 *
 * @section history Change History
 * - v1.0 (2021-12-28)
 *   - First release

 */

#include "usblini.h"
#include "usb.h"
#include "clock.h"
#include "led.h"
#include "bootloader.h"
#include "lin.h"
#include "linmaster.h"
#include "linslave.h"
#include "statusreport.h"


/**
 * @brief Main function and entry point of application
 * @param argc Count of arguments
 * @param argv Argument vector
 * @return Error code
 */
int main(int argc, char** argv) {

    // set clock to 3x PLL, 48 MHz
    OSCCON = 0b11111100;

    // activate active clock tuning with USB clock
    ACTCON = 0b10010000;

    // wait until clock is stable
    while ((OSCSTATbits.HFIOFR == 0) &&
            (OSCSTATbits.PLLRDY == 0));

    ANSELC = 0x00;
    ANSELA = 0x00;
    APFCON = 0x40;

    // set default output levels
    LATC = 0x00;
    
    // set in-/output
    TRISA = 0x00;
    TRISC = 0x23;
    
    // initialize software modules
    clock_init();
    led_init();
    usb_init();
    lin_init();
    linslave_init();
    statusreport_init();

    // enable interrupts
    PEIE = 1; // peripheral interrupt enable
    GIE = 1; // enable global interrupts

    
    // SPI
    // RC0/ICSPDAT   CLK
    // RC1/ICSPCLK   IN
    // RA4           OUT
        
    PR2 = 119; 
    T2CON = 0x04;
    PWM2DCL = 0;    
    PWM2DCH = 59;
    PWM2CON = 0xC0;
        
    SSPSTAT = 0x00;
    SSPCON1 = 0x05; // SPI slave
    SSP1BUF = 0x00;
    SSPCON1bits.SSP1EN = 1;
    SSP1IF = 0;
    SSP1IE = 1;
    
    uint16_t timestamp = clock_getMSTicker();
    
    // main loop
    while (1) {

        // reset / bootloader
        if (!PORTAbits.RA3) {
            usb_shutdown();
            asm("ljmp 0x000");
        }
        
        if (clock_diff(timestamp) > 1000) {
            timestamp = clock_getMSTicker();

            // for user functions: do some periodic task (1 Hz)
            
        }

        // do module processing
        usb_process();
        bootloader_process();  
        lin_process();
        linslave_process();
        linmaster_process();
        statusreport_process();
    };

    return 0;
}

/**
 * @brief Interrupt service routine
 */
void __interrupt() isr(void) {

    // usart rx interrupt
    if (RCIE && RCIF) {        
        lin_readByte();
    }
    
    // spi interrupt
    if (SSP1IE && SSP1IF) {
        usb_ep2_put(SSP1BUF);
        SSP1IF = 0;
        lin_timeout_isr();
    }
    
    // timer 1 interrupt
    if (TMR1IE && TMR1IF) {
        TMR1IF = 0;

        clock_isr();
        led_isr();
        linmaster_timer_isr();
    }
}