/**
 * @file statusreport.c
 *
 * @brief This file contains status report routines for the USBlini firmware
 *        project.
 *
 * @author Thomas Fischl (https://www.fischl.de/contact.html)
 * @copyright (c) 2021 Thomas Fischl
 * 
 * This file is part of the USBlini firmware project and is copyrighted by the
 * authors listed above. It is free for private or educational non-commercial use
 * (for a commercial license please contact the authors). It may not be
 * redistributed without the prior written consent of the authors.
 *
 * It is provided "as is" without warranty of any kind, either express or implied,
 * including without limitation any implied warranties of condition, uninterrupted
 * use, merchantability, fitness for a particular purpose, or non-infringement. In
 * no event shall the authors be liable for any direct or indirect damages arising
 * in any way out of the use of it.
 *
 */

#include "usblini.h"
#include "lin.h"
#include "linslave.h"
#include "usb.h"
#include "statusreport.h"

#define STATUSREPORT_SIZE 16
typedef struct
{
    uint8_t reportid;
    uint8_t errorflags;
    uint16_t slaveTableStatus;
    uint8_t reserved[STATUSREPORT_SIZE - 4];
} statusreport_t;

volatile statusreport_t statusreport;
volatile uint8_t hasChanged;

/**
 * @brief Initialize status report module
 */
void statusreport_init(void) {
    statusreport.reportid = REPORT_TYPE_STATUS;
    statusreport.errorflags = 0;
    statusreport.slaveTableStatus = 0;
    hasChanged = 0;
}

/**
 * @brief Reset status report module
 */
void statusreport_reset(void) {
    
    uint8_t i;
    for (i = 0; i < SLAVETABLE_SIZE; i++) {
        statusreport_setSlaveTableStatus(i, 0);
    }
    
    statusreport_clearErrorflags(0xff);    
}

/**
 * @brief Set given error flag
 * @param flag Error flag to set
 */
void statusreport_setErrorflag(uint8_t flag) {
    uint8_t newerrorflags = statusreport.errorflags | flag;
    
    if (newerrorflags != statusreport.errorflags) {
        statusreport.errorflags = newerrorflags;
        hasChanged = 1;        
    }
}

/**
 * @brief Clear error flags using a bit mask
 * @param mask Bit mask of errors to clear
 */
void statusreport_clearErrorflags(uint8_t mask) {
    
    uint8_t newerrorflags = statusreport.errorflags & ~mask;
    
    if (newerrorflags != statusreport.errorflags) {
        statusreport.errorflags = newerrorflags;
        hasChanged = 1;        
    }    
}

/**
 * @brief Set slave table status
 * @param tableid Table id
 * @param status Status to set
 */
void statusreport_setSlaveTableStatus(uint8_t tableid, uint16_t status) {
    uint16_t newstatus;
    
    if (status) {
        newstatus = statusreport.slaveTableStatus | (1 << tableid);
    } else {
        newstatus = statusreport.slaveTableStatus & ~(1 << tableid);
    }
    
    if (newstatus != statusreport.slaveTableStatus) {
        statusreport.slaveTableStatus = newstatus;
        hasChanged = 1;        
    }
}

/**
 * @brief Do module processing
 */
void statusreport_process(void) {
    
    if (hasChanged) {
        usb_ep1_put((uint8_t*)&statusreport, STATUSREPORT_SIZE);
        hasChanged = 0;
    }
    
}